`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    18:13:48 12/18/2008 
// Design Name: 
// Module Name:    MOD-DEV-100-CPLD-R1p3 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module MOD_DEV_100_CPLD_R1p4(
    inout [7:0] DataBus,
    input [2:0] AddrBus,
	 input nCS3,
	 input RnW,
    input CPU_CLK,
	 input [0:7] Switches, 
    output reg [8:1] LEDs = 8'hFF,
    output [7:0] SevenSegDat,
    output reg[4:1] SevenSegSel,
    inout nIRQ1,
    input nIRQSwitch_RTC,
    input nRSTI,
    input wire nRSTO,
    inout SD_CS_DAT3,
    inout SD_DI_CMD,
    inout SD_DO_DAT0,
    inout SD_IRQ_DAT1,
    inout SD_RW_DAT2,
    inout SD_SCLK_CLK
    );
	parameter Dev_100_CPLD_Version = 8'd1;
	parameter Dev_100_CPLD_Revision = 8'd4;
	parameter SevSegCountSize = 12;
		
	wire nReset = ( nRSTI & nRSTO );
	
	reg [7:0]DataOut;
	
	reg [(SevSegCountSize-1):0]SevSegCount = 0;
	wire [1:0] digitScan = SevSegCount[(SevSegCountSize-1):(SevSegCountSize-2)];
	
	reg [4*4-1:0] BCD_digits;
	reg [3:0] BCDdigit = 16'hFFFF;
	reg [7:0]SevenSeg;
	reg [1:4] DecimalPlace;
	reg ManualControl;
	reg intRnW = 1'b0;
	
 
// If irq button is asseted drive cpu irq low, else hiz 
	 assign nIRQ1 = nIRQSwitch_RTC ? 1'bz : nIRQSwitch_RTC;
	 
// assign correct 7 segment data to active display
	assign SevenSegDat = ~(SevenSeg | ( (SevenSegSel && DecimalPlace[digitScan+1] )<<7) );
	
	
//Reading on the databus
	assign DataBus = ((nCS3==1) || (intRnW==0)) ? 8'BZ : DataOut;

	always @ ( nCS3 )
	begin
		case( AddrBus )
			3'd0: DataOut = Switches;
			3'd1: DataOut = BCD_digits[ 15 : 8 ];
			3'd2: DataOut = BCD_digits[ 7 : 0 ];
			3'd3: DataOut = { ManualControl, 3'b000, DecimalPlace };
			3'd4: DataOut = ~LEDs;
			3'd5: DataOut = Dev_100_CPLD_Version;
			3'd6: DataOut =  Dev_100_CPLD_Revision;
			default:  DataOut = Switches;
		endcase
	end


//Reeseting registers or writing to the data bus
	always @ (  posedge nCS3 or negedge nRSTO  )
	begin
	
	if( !nRSTO )
		begin
			BCD_digits = 16'hFFFF;
			LEDs = 8'hFF;
		end
		
	else 
		begin
			if( !intRnW ) 
			begin
			case( AddrBus )
				0: LEDs = ~DataBus;
				1: BCD_digits[15:8] = DataBus;
				2: BCD_digits[7:0] = DataBus;
				3: { ManualControl, DecimalPlace[1:4] } = { DataBus[7], DataBus[3:0] };
				default:  LEDs = ~DataBus;
			endcase
			end
		end
	end
	

//Convert BCD to 7 segment and rotate active display
	always @ ( posedge CPU_CLK ) 
	begin
	
	SevSegCount <= (SevSegCount + 1);
	intRnW <= RnW;	
	
	if( ManualControl )
	begin
		SevenSegSel = BCD_digits[ 11: 8 ];
		SevenSeg = BCD_digits[ 7 : 0 ];
	end
	else
	begin
	
	// rotate active 7 segment diplay
	SevenSegSel = (1<<digitScan);
	
	case(digitScan)
		 0: BCDdigit = BCD_digits[ 15: 12 ];
		 1: BCDdigit = BCD_digits[ 11: 8 ];
		 2: BCDdigit = BCD_digits[ 7 : 4 ];
		 3: BCDdigit = BCD_digits[ 3 : 0 ];
	endcase

	case(BCDdigit)
		0: SevenSeg = 8'b00111111;
		1: SevenSeg = 8'b00000110;
		2: SevenSeg = 8'b01011011;
		3: SevenSeg = 8'b01001111;
		4: SevenSeg = 8'b01100110;
		5: SevenSeg = 8'b01101101;
		6: SevenSeg = 8'b01111100;
		7: SevenSeg = 8'b00000111;
		8: SevenSeg = 8'b01111111;
		9: SevenSeg = 8'b01100111;
		default: SevenSeg = 8'b00000000;
	endcase
	end
	end
	
		


endmodule


